(module faces (lib "slideshow.ss" "texpict")
  (require (lib "mred.ss" "mred")
           (lib "class.ss")
           (lib "etc.ss")
           (lib "math.ss")
	   "bmhelp.ss")

  (provide face series)

  (define face-color (make-object color% "orange"))
  (define face-bright-edge-color (scale-color #e1.6 face-color))
  (define face-edge-color (scale-color #e0.8 face-color))
  (define face-hard-edge-color (scale-color #e0.8 face-edge-color))
  
  (define no-brush (find-brush "white" 'transparent))
  (define no-pen (find-pen "white" 1 'transparent))

  (define (series dc steps start-c end-c f pen? brush?)
    (color-series dc steps #e0.5 start-c end-c f pen? brush?))
  
  (define (face mood)
    (let ([w 300]
          [h 300])
      (dc (lambda (dc x y)
            (define old-pen (send dc get-pen))
            (define old-brush (send dc get-brush))
            
            (define (one-eye l? p? dd)
              (define s (if p? 1/3 1))
              (define sdd (if p? 1 1/2))
              (define dx (if p? (* 1/5 w 1/3) 0))
              (define dy (if p? (* 1/4 w 1/3) 0))
              (send dc draw-ellipse
                    (+ x (* w (if l? 1/5 3/5)) dx (* dd sdd)) (+ y (* h 1/5) dy (* dd sdd))
                    (- (* w 1/5 s) (* 2 dd)) (- (* h 1/4 s) (* 2 dd))))
            
            (define (one-eye-brow l? dd dy dr)
              (send dc draw-arc
                    (+ x (* w (if l? 1/5 3/5))) (+ y (* h 3/20) dd (- dy))
                    (* w 1/5) (* h 1/4)
                    ((if l? + -) (* pi 1/3) dr) ((if l? + -) (* pi 2/3) dr)))
            
            (define (eye-series steps start-c end-c p? extra-inset)
              (series dc steps start-c end-c
                      (lambda (i)
                        (one-eye #t p? (+ extra-inset i))
                        (one-eye #f p? (+ extra-inset i)))
                      #f #t))
            
            (define (eyebrows dy dr)
              (send dc set-brush no-brush)
              (series dc 3
                      face-hard-edge-color
                      face-edge-color
                      (lambda (i)
                        (one-eye-brow #t i dy dr)
                        (one-eye-brow #f i dy dr))
                      #t #f)
              (send dc set-pen no-pen))
            
	    (define (normal-eyebrows)
	      (eyebrows 0 0))

	    (define (raised-eyebrows)
	      (eyebrows 3 0))

	    (define (worried-eyebrows)
	      (eyebrows 0 0.3))

            (define (smile sw sh i da rgn dy flip?)
              ((if rgn 
                   (lambda (x y w h s e)
                     (send rgn set-arc x y w h s e))
                   (lambda (x y w h s e)
                     (send dc draw-arc x y w h s e)))
               (+ x (/ (- w sw) 2) (* 1/6 sw)) 
               (+ y (/ (- sh h) 2) (* 1/8 h) dy (if flip? i 0) (if flip? (- (* h 1/2) (- sh h)) 0))
               (* sw 2/3) (+ (if flip? 0 i) (* h 2/3))
               (- (* pi (- 5/4 (if flip? 1 0))) da) (+ (* pi (- 7/4 (if flip? 1 0))) da)))
            
            (define (plain-smile flip?)
              (send dc set-brush no-brush)            
	      (series dc 3
                      (make-object color% "black")
                      face-edge-color
                      (lambda (i)
                        (smile w h i 0 #f 0 flip?)
                        (smile w h (- i) 0 #f 0 flip?))
                      #t #f))
            
            (define (teeth)
              ;; Assumes clipping region is set
              (send dc set-brush (find-brush "white"))
              (send dc draw-ellipse x y w h)
              (series dc 5
                      (make-object color% "darkgray")
                      (make-object color% "lightgray")
                      (lambda (i)
                        (let loop ([j 0][delta 0][tw (* w 1/10)])
                          (unless (= j 5)
                            (send dc draw-rectangle
                                  (+ x (* w 1/2) delta 1) y
                                  (- tw i 1) h)
                            (send dc draw-rectangle
                                  (+ x (* w 1/2) (- delta) (- tw) 1) y
                                  (- tw i 1) h)
                            (loop (add1 j) (+ delta tw) (* 8/10 tw)))))
                      #f #t))
            
            (define (toothy-smile tw th ta bw bh ba flip?)
              (let-values ([(tmp-rgn1) (make-object region% dc)]
                           [(tmp-rgn2) (make-object region% dc)]
                           [(dy) (/ (- h (if flip? (+ th (abs (- bh th))) th)) 2)])
                ;; Teeth:
                (smile tw th 0 ta tmp-rgn1 dy flip?)
                (smile bw bh 0 ba tmp-rgn2 dy flip?)
                (send tmp-rgn1 subtract tmp-rgn2)
                (send dc set-clipping-region tmp-rgn1)
                (teeth)
                (send dc set-clipping-region #f)
                
                ;; Smile edges:
                (send dc set-brush no-brush)
                (series dc 3
                        (if flip? face-bright-edge-color face-hard-edge-color)
                        (if flip? face-color face-edge-color)
                        (lambda (i)
                          (smile bw bh (if flip? i (- i)) ba #f dy flip?))
                        #t #f)
                (series dc 3
                        (if flip? face-hard-edge-color face-bright-edge-color)
                        (if flip? face-edge-color face-color)
                        (lambda (i)
                          (smile tw th (if flip? (- i) i) ta #f dy flip?))
                        #t #f)))
            
            (define (grimace tw th ta flip?)
              (let-values ([(tmp-rgn1) (make-object region% dc)]
                           [(tmp-rgn2) (make-object region% dc)]
                           [(dy) (/ (- h th) 2)]
                           [(elx ely) (values (+ x (* w 0.27)) (+ y (* h 0.65) (if flip? 3 1)))])
                ;; Teeth:
                (smile tw th 0 ta tmp-rgn1 (+ (if flip? -30 0) dy) flip?)
                (smile tw th 0 ta tmp-rgn2 (+ (if flip? 0 -30) dy) flip?)
                (send tmp-rgn1 subtract tmp-rgn2)
                (send tmp-rgn2 set-ellipse elx ely 30 30)
                (send tmp-rgn1 union tmp-rgn2)
                (send tmp-rgn2 set-ellipse (- (+ x w) (- elx x) 30) ely 30 30)
                (send tmp-rgn1 union tmp-rgn2)
                (send dc set-clipping-region tmp-rgn1)
                (teeth)
                (send dc set-clipping-region #f)
                
                ;; Smile edges:
                (send dc set-brush no-brush)
                (let ([sides (lambda (top? i)
                               (send dc draw-arc (- elx (/ i 2)) (- ely (/ i 2)) 30 (+ 30 i) 
                                     (* pi (if top? 1 1/2)) (* pi (if top? 3/2 1)))
                               (send dc draw-arc (+ (- (+ x w) (- elx x) 30) (/ i 2)) (- ely (/ i 2)) 30 (+ 30 i) 
                                     (* pi (if top? -1/2 0)) (* pi (if top? 0 1/2))))])
                  (series dc 3
                          (if flip? face-bright-edge-color face-hard-edge-color)
                          (if flip? face-color face-edge-color)
                          (lambda (i)
                            (sides flip? i)
                            (smile tw th (if flip? i (- i)) ta #f (+ (if flip? -2 -30) dy) flip?))
                          #t #f)
                  (series dc 3
                          (if flip? face-hard-edge-color face-bright-edge-color)
                          (if flip? face-edge-color face-color)
                          (lambda (i)
                            (sides (not flip?) i)
                            (smile tw th (if flip? (- i) i) ta #f (+ (if flip? -30 0) dy) flip?))
                          #t #f))))
            
            (define (medium-grimace flip?)
              (grimace
               (* 1.2 w) (* h 0.9) (- (* 0.1 pi))
               flip?))
            
            (define (large-smile flip?)
              (toothy-smile
               w (* 1.05 h) (* 0.10 pi)
               (* 1.1 w) (* h 0.95) (* 0.05 pi)
               flip?))
            
            (define (largest-smile flip?)            
              (toothy-smile
               w (* 1.1 h) (* 0.14 pi)
               (* 1.2 w) (* h 0.9) (* 0.05 pi)
               flip?))
            
            (define (narrow-smile flip?)
              (toothy-smile
               (* 0.8 w) (* h 0.7) (- (* 0.00 pi))
               (* 1.0 w) (* h 0.6) (- (* 0.06 pi))
               flip?))
            
            (define (medium-smile flip?)
              (toothy-smile
               (* 0.8 w) (* h 0.9) (* 0.08 pi)
               (* 1.0 w) (* h 0.75) (- (* 0.01 pi))
               flip?))
            
              
            (define (draw-eyes inset)
              ;; Draw eyes
              (eye-series 10 
                          (make-object color% "lightgray")
                          (make-object color% "white")
                          #f
                          inset)
              
              ;; Draw pupils
              (eye-series 3
                          (make-object color% 220 220 220)
                          (make-object color% "black")
                          #t
                          0))

            (send dc set-pen no-pen)

            ;; Draw face background
            (series dc 5
                    face-edge-color
                    face-color
                    (lambda (i)
                      (send dc draw-ellipse 
                            (+ x (/ i 2)) (+ y (/ i 2))
                            (- w (* 2 i)) (- h (* 2 i))))
                    #f #t)
            
            (case mood
              [(unhappy)
               (draw-eyes 6)
               (plain-smile #t)]
              [(sortof-happy)
               (draw-eyes 6)
	       (worried-eyebrows)
               (medium-grimace #f)]
              [(sortof-unhappy)
               (draw-eyes 6)
	       (worried-eyebrows)
               (medium-grimace #t)]
              [(happy)
               (draw-eyes 6)
               (plain-smile #f)]
              [(happier)
               (draw-eyes 3)
               (large-smile #f)]
              [(embarassed)
               (draw-eyes 3)
               (worried-eyebrows)
               (medium-smile #f)]
              [(badly-embarassed)
               (draw-eyes 3)
               (worried-eyebrows)
               (medium-smile #t)]
              [(unhappier)
               (draw-eyes 3)
               (normal-eyebrows)
               (large-smile #t)]
              [(happiest)
               (draw-eyes 0)
               (raised-eyebrows)
               (largest-smile #f)]
              [(happiest-plain)
               (draw-eyes 0)
               (largest-smile #f)]
              [(unhappiest)
               (draw-eyes 0)
               (raised-eyebrows)
               (largest-smile #t)])
            
            (send dc set-brush old-brush)
            (send dc set-pen old-pen))
          w h 0 0))))
